
try:
    import libhid
    pacdrive_use_libhid = True
except:
    pacdrive_use_libhid = False

import os
import re


#################################################################
# The shared elements for pacdrive classes
#################################################################
class pd_root:
    # Current state of LEDs
    initialized = False
    debugstr = ""

    # Get the state of all 16 LEDs (16 bit value returned)
    def get_led_state(self, boardnum):
        if self.initialized:
            return self.led_state[boardnum]
        else:
            return -1
  
    # Set the state of all 16 LEDs (pass in a 16 bit value)
    # boardnum starts at 0 and goes up to (self.get_count-1)
    def set_led_state(self, newstate, boardnum):
        pass

    # Not to be called directly (private)
    def init_base(self):
        self.initialized = True
        self.debugstr = ""
        self.min_led_num = 1
        self.max_led_num = 16
        self.max_state_val = 0xffff
        self.max_pacs = 5

        # Led state is a list of states
        self.led_state = []
        for idx in range(0, self.max_pacs):
            self.led_state.append(0)

        self.pac_count = 0

    # Must call before and set/get accesses
    def init(self):
        self.init_base()

    # Returns the number of pac-drives found
    def get_count(self):
        if self.initialized:
           return self.pac_count
        else:
           return 0

    # Opposite of init -- call after last access
    def cleanup(self):
        pass

    # Turn on/off a single LED.  lednum should be between 1-16
    # state should be 1/0 or true/false
    # NOTE: you MUST make a call to set_led_state before this will work properly
    def set_single_led(self, lednum, state, boardnum):
        if ((lednum >= self.min_led_num) and (lednum <= self.max_led_num)):
            newstate = self.get_led_state(boardnum)
            mask = 1 << (lednum - 1)
            #print "SSL: cur state: " + `hex(newstate)`
            if (state):
                # turning LED ON
                newstate |= mask
            else:
                # turning LED OFF
                newstate &= ~mask

            #print "SSL: new state: " + `hex(newstate)`
            return self.set_led_state(newstate, boardnum)
        else:
            # invalid LED num
            return -1

    # Check the state of a single LED
    # Returns: true if on, else false
    def get_single_led(self, lednum, boardnum):
        retval = -1
        if ((lednum >= self.min_led_num) and (lednum <= self.max_led_num)):

            curstate = self.get_led_state(boardnum)
            mask = 1 << (lednum - 1)
            #print "GSL: bit mask is: " + `mask`
            #print "GSL: cur state: " + `hex(curstate)`
            if (mask & curstate):
                print "returning true"
                retval = True
            else:
                print "returning false"
                retval = False
       
        return retval

    def dprint(self, str):
        # for temp debug.  Later just store in str
        #print str
        self.debugstr += str;

#################################################################
# Pacdrive object -- use appropriate interface to talk to Pac-Drive
#################################################################
class pacdrive(pd_root):
    child = None

    def __init__(self, intf=""):

        if (intf == ""):
            # If not interface specified, try to 
            # use libhid if possible
            intf = "external"
            if pacdrive_use_libhid:
                intf = "libhid"
        elif (intf == "libhid"):
            # User specified libhid
            if not pacdrive_use_libhid:
                # don't have libhid... not going to work
                intf = "external"
        else:
            # User specified external or something invalid
            # default is external
            intf = "external"

        self.initialized = False
        if (intf == "libhid"):
            self.dprint("using libhid")
            self.child = pacdrive_libhid()
        else:
            self.dprint("using pacdrive program")
            self.child = pacdrive_external()

    def get_led_state(self, boardnum=0):
        return self.child.get_led_state(boardnum)

    def set_led_state(self, newstate, boardnum=0):
        return self.child.set_led_state(newstate, boardnum)

    def set_single_led(self, lednum, boardnum=0):
        return self.child.get_single_led(lednum, boardnum)

    def set_single_led(self, lednum, state, boardnum=0):
        return self.child.set_single_led(lednum, state, boardnum)

    def init(self):
        return self.child.init()

    def cleanup(self):
        self.child.cleanup()

    def get_count(self):
        return self.child.get_count()

#################################################################
# Access pacdrive directly via libhid
# NOTE: this isn't coded yet (obviously), so it would be good
# to specify "external" when creating a pacdrive object
#################################################################
class pacdrive_libhid(pd_root):

    # Note: boardnum is currently ignored and assumed to be 0
    def open_pac(self, boardnum):
        hid = hid_new_HIDInterface()
        matcher = HIDInterfaceMatcher()
        matcher.vendor_id  = 0xd209
        matcher.product_id = 0x1500

        hid_force_open(hid, 0, matcher, 3)
        if ret != HID_RET_SUCCESS:
            print "Failed to open pac-drive: " + `boardnum`
            hid_cleanup()
            return None
        else:
            return hid
    
    def set_led_state(self, newstate, boardnum):

        retval = -1
        hid = self.open_pac(boardnum)
        if (hid != None):
            if ((self.initialized) and 
                (newstate <= self.max_state_val) and 
                (newstate >= 0)):

                # TODO: send output report here
                path_in = ""
                path = ""
                data = ""
                ret = hid_set_output_report(hid, path_in, 1, data, 4)
                print "TODO: use libhid to set LED state"
                self.led_state[boardnum] = newstate
                retval = 1

        return retval

    # Returns count of devices found
    def find_pacs(self):
        hid_set_debug(0)
        hid_set_usb_debug(0)
        ret = hid_init()
        if ret != HID_RET_SUCCESS:
            print "failed to init libhid"
            return -1

        # TODO: implement this
        # The python bindings don't provide the
        # full matcher with callback, so it might not be possible
        # to implement the counting function directly -- use external

        # Forcing to only 1 device for now
        return 1

    def init(self):
        self.init_base()

        self.pac_count = self.find_pacs()
        self.pac = None

        # No way to find a particular device via libhid...
        # In this case, the product and vendor ids are the same
        # (and that seems to be the case for serial num too)
   
        return self.pac_count

    def cleanup(self):
        self.initialized = False
        hid_cleanup()


#################################################################
# Access pacdrive via external pacdrive program
#################################################################
class pacdrive_external(pd_root):
    pacdrive_pgm_only = "pacdrive"
    pacdrive_pgm = os.path.join("/usr/bin", pacdrive_pgm_only)
    cmd_which = "which"

    def find_pacdrive_prog(self):
        if not os.path.exists(self.pacdrive_pgm):
            self.pacdrive_pgm = ""
            # first check the current dir
            if os.path.exists("./" + self.pacdrive_pgm_only):
                self.pacdrive_pgm = "./" + self.pacdrive_pgm_only
            else:
                # try to locate and set the pacdrive program
                for file in os.popen(self.cmd_which + ' ' + self.pacdrive_pgm_only).readlines():
                    self.pacdrive_pgm = file.rstrip()

        self.dprint("Pacdrive program is: " + self.pacdrive_pgm)

        return self.pacdrive_pgm

    def force_two_byte(self, someval):
        # Kind of a brutal way to force a 16 bit value
        hexstr = hex(someval)
        #self.dprint("force two: hexstr: " + hexstr)
        # 6 characters to include 0x
        if len(hexstr) != 6:
            #self.dprint("force two: not 6 byte")
            hexstr = hexstr[2:]
            #self.dprint("force two: stripped 0x: " + hexstr)
            while len(hexstr) < 4:
                hexstr = "0" + hexstr
            #self.dprint("force two: added leading 0: " + hexstr)
            hexstr = "0x" + hexstr[-4:]
            #self.dprint("force two: added 0x:" + hexstr)

        return hexstr

    def set_led_state(self, newstate, boardnum):
        retval = -1
        self.dprint("pd_ext: entering set_led_state")
        if ((self.initialized) and 
           (newstate <= self.max_state_val) and 
           (newstate >= 0)):

            done_re = "^done"
            hexval = self.force_two_byte(newstate)
            self.dprint("from force: " + hexval)
            
            cmd = self.pacdrive_pgm + " -s " + hexval + " -b " + `boardnum`
            self.dprint("Set_led_state: Cmd: " + cmd)

            # call pacdrive program to try to set the LEDs
            for line in os.popen(cmd).readlines():
                self.dprint("set_led_state: line: " + line)
                res = re.match(done_re, line)
                if res:
                    self.dprint("matched done")
                    self.led_state[boardnum] = newstate
                    retval = 1
                    break
        
        self.dprint("pd_ext: set returning: " + `retval`)
        return retval

    # Returns the number of pac-drives found
    def find_pacs(self):
        retval = 0
        if (self.initialized):
            count_re = "^Detect: found ([0-9]+) Pac-Drives"

            cmd = self.pacdrive_pgm + " -d"
            self.dprint("find_pacs: Cmd: " + cmd)
            for line in os.popen(cmd).readlines():
                self.dprint("set_led_state: line: " + line)
                res = re.match(count_re, line)
                if res:
                    retval = int(res.group(1))
                    break

        return retval

    def init(self):
        if self.find_pacdrive_prog() == "":
            return -1
        self.dprint("init: Creating pacdrive_external object")
        self.init_base()

        self.pac_count = self.find_pacs()
        return 1

    def cleanup(self):
        self.initialized = False



