/*************************************************************************
 * TLV 10/11/2007                                                        *
 * Copyright (C) 2007 Travis Veldkamp                                    *
 *                                                                       *
 * This program is for controlling the Pac-Drive LED controller          *
 * For more information, see http://www.ultimarc.com                     *
 *                                                                       *
 * This is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * at your option) any later version.                                    *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with Foobar; if not, write to:                                  *
 *                   Free Software Foundation, Inc.                      *
 *                   59 Temple Place                                     *
 *                   Suite 330                                           *
 *                   Boston, MA  02111-1307                              *
 *                   USA                                                 *
 *************************************************************************/


#include "cmd_line.h"
#include "pac_prog.h"
#include <signal.h>
#include <stdlib.h>
#include "string.h"
#include "util.h"


static const int MAJOR_VERSION = 0;
static const int MINOR_VERSION = 1;
static const int VERSION_DATE  = 20071011;

/*************************************************************************
 * GLOBAL VARIABLES                                                      *
 *************************************************************************/

/*This is set by the signal handler (sig_handle()) when sig_int is caught*
 *This allows us to clean up before falling out                          */
int want_out=0;

/*************************************************************************
 * END GLOBAL VARIABLES                                                  *
 *************************************************************************/



/***********************************************************
 * Simple signal handler to set global variable indicating *
 * that the user wants out, but allows us time to clean up *
 ***********************************************************/
void sig_handle(int signal) {

   if (signal == SIGINT) {
      printf("Exiting.... please wait while cleaning up\n");
      want_out = 1;
   }
}

void PrintWelcome() {
   printv(VERBOSE_VERBOSE, "pacdrive v%d.%02d - %d\n", MAJOR_VERSION,
           MINOR_VERSION, VERSION_DATE);
   printv(VERBOSE_VERBOSE, "Pac-Drive LED controller utility for Linux\n");
   printv(VERBOSE_VERBOSE, "Copyright (C) 2007 Travis Veldkamp\n\n");
}

void ShowUsage() {
   PrintWelcome();

   cmdline_ShowUsage();
}

int main(int argc, char **argv) {

   int error=0;
   int idx;
   struct cmdline_struct settings;
   int numpacs;
   PacData pacdata;
   HIDInterface *pac;

   openlog("pacdrive.log");
   memset(&pacdata, 0, sizeof(PacData));

   if (parse_cmdline(argc, argv, &settings) < 0) {
      settings.showhelp = 1;
   }

#ifdef DEBUG
   printf("verbose: %d\n", settings.verbose);
   printf("showhelp: %d\n", settings.showhelp);
   printf("board_number: %d\n", settings.board_number);
   printf("detection: %d\n", settings.run_detection);
   printf("no program: %d\n", settings.no_program);
   printf("get current: %d\n", settings.get_current);
   printf("byte 1: 0x%x\n", settings.ledbytes[0]);
   printf("byte 2: 0x%x\n", settings.ledbytes[1]);
#endif

   if (settings.showhelp) {
      ShowUsage();
      clean_exit(1);
   }

#ifdef FORCEROOT
   /* TODO: Does this work if it's set SUID? */
   if (getuid()) {
      printe("This program can only be run as root\n");
      clean_exit(1);
   }
#endif

   /*Register the signal handler for sig int (ctrl-c)*/
   if (signal(SIGINT, &sig_handle) == SIG_ERR) {
      printl(VERBOSE_VERBOSE,
             "Error registering new signal handler\n");
      error = 1;
      want_out = 1;
   }

   /* Find the Pac-Drives */
   numpacs = find_pacs(MAX_PACS);
   if (numpacs < 1) {
      printe("Failed to find a Pac-Drive.  Is it plugged in?  Do you have permissions?\n");
      clean_exit(1);
   }

   if (settings.run_detection) {

      if (settings.verbose) PrintWelcome();

      printv(VERBOSE_VERBOSE, "Attempting to detect Pac-Drive LED controllers\n");

      printv(VERBOSE_NORMAL, "Detect: found %d Pac-Drives\n", numpacs);
      
      settings.no_program = 1;
   }

   if (settings.no_program) {
      /* User has asked for no programming step*/
      /* This is most likely used when the user wants to detect their board*/
      printl(VERBOSE_NORMAL,
             "No programming cycle requested\n");
      printl(VERBOSE_VERBOSE, "Exiting.\n");
      want_out = 1;
   }

   if ((!error) && (!want_out)) {
      if (settings.verbose) PrintWelcome();

   }

   if (settings.board_number >= numpacs) {
      printe("Board %d specified, but only found %d Pac-Drive(s).  Numbering starts at 0\n", settings.board_number, numpacs);
      clean_exit(1);
   }

   /* Now, try to program the PAC-Drive */
   if ((!error) && (!want_out)) {
      printl(VERBOSE_VERBOSE, "Programming Pac-Drive... " );

   } else {
      clean_exit(1);
   } 

   /* Init the one we're using */
   pac = init_pac(settings.board_number);
   if (pac == NULL) {
       printe("Failed to initialize Pac-Drive.  (do you have permissions for the device?)\n");
       clean_exit(1);
   }

   // TODO: does the board support this?  If so, add back in
   /* get the current settings */
   //error = get_led_state(pac, &pacdata);
   //printl(VERBOSE_VERBOSE, "get_led_state: returned: %d\n", error);
   //settings.ledbytes[0] = pacdata.Data[1];
   //settings.ledbytes[1] = pacdata.Data[2];
   //printl(VERBOSE_VERBOSE, "led state after get_led_state: 0x%x%x\n", pacdata.Data[1], pacdata.Data[2]);

   /* re-parse the cmd line -- some options depend on current LED state */
   // TODO: if get current works, add this back in
//   parse_cmdline(argc, argv, &settings);

   for(idx=0; idx < PAC_DATA_DATA_LEN; idx++) {
       pacdata.Data[idx] = settings.ledbytes[idx];
   }
   printl(VERBOSE_VERBOSE, "led state before set: 0x%x%x\n", pacdata.Data[1], pacdata.Data[2]);

   /* Finally, set the new states */
   error = set_led_state(pac, &pacdata);
   printl(VERBOSE_VERBOSE, "set_led_state: returned: %d\n", error);

   close_pac(pac);
   closelog();
   
   if (error) {
      printe("** Programming FAILED **\n");
      return 1;
   } else {
      printv(VERBOSE_VERBOSE, "** PAC-Drive has been reprogrammed **\n");
      printv(VERBOSE_NORMAL, "done.\n");
      return 0;
   }

}
