#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "pac_prog.h"
#include "util.h"

static int gnum_pacs_found;
static int temp_pac_count;

/* This always returns false, but increments countvar */
bool match_fn_count(struct usb_dev_handle *usbdev, void *countvar, int maxnum) {

   int *count = (int *)countvar;

   if ((usb_device(usbdev)->descriptor.idVendor == VENDOR_ID_ULTIMARC) &&
       (usb_device(usbdev)->descriptor.idProduct == PRODUCT_ID_PACDRIVE)) {

       //printf("Found one device\n");
       if (*count < maxnum) {
           *count = *count + 1;
       }
   }
  
   return 0;
}

/* This increments countvar until it matches boardnum and then returns true */
bool match_fn_find_selected(struct usb_dev_handle *usbdev, void *countvar, int boardnum) {
   bool retval = 0;
   int *count = (int *)countvar;

   if ((usb_device(usbdev)->descriptor.idVendor == VENDOR_ID_ULTIMARC) &&
       (usb_device(usbdev)->descriptor.idProduct == PRODUCT_ID_PACDRIVE)) {

       if (*count == boardnum) {
           retval = 1;
       }
       *count = *count + 1;
   }

   return retval;
}

/* Sets the devices in the *dev_arr */
/* Returns the number of devices found */
int find_pacs(int max_num) {
  hid_return ret;
  HIDInterface *pac;
  HIDInterfaceMatcher matcher = { VENDOR_ID_ULTIMARC,
                                  PRODUCT_ID_PACDRIVE,
                                  (void *)match_fn_count,
                                  &gnum_pacs_found,
                                  max_num
                                };
  gnum_pacs_found = 0;
                 

  //hid_set_debug(HID_DEBUG_ALL);
  hid_set_debug(0);
  hid_set_debug_stream(stderr);
  hid_set_usb_debug(0);
  
  // hid_init does usb_init, find busses, find devices
  ret = hid_init();
  if (ret != HID_RET_SUCCESS) {
    printl(0, "hid_init failed with return code %d\n", ret);
    return -1;
  }
 
  // This does a malloc under the covers.  
  // Make sure to hid_delete_HIDInterface()... 
  pac = hid_new_HIDInterface();
  if (pac == NULL) {
      printl(0, "find_pacs(): failed to get a new HID interface... d'oh\n");
      return 0;
  }

  // This is kind of a hack... we want to find how many Pac-Drives are attached, but libhid
  // doesn't provide an easy way to do it.  So, we'll call open with a custom matcher, callback fn
  // This callback will never find a match, but will increment the count variable
  // SO, This open MUST fail.  We pass in a matcher with a function that never finds the device
  
  ret = hid_force_open(pac, 0, &matcher, 1);

  //printf("After open, count: %d\n", gnum_pacs_found);
  
  if (ret == HID_RET_SUCCESS) {
      printl(0, "find_pacs(): hid_force_open found a device.  D'oh\n");
      hid_delete_HIDInterface(&pac);
      return 0;
  }
  hid_delete_HIDInterface(&pac);

  //printf("find_pacs(): returning: %d\n", gnum_pacs_found);
  return gnum_pacs_found;

}

/* Set LED states by sending data to Pac-Drive */
/* Returns 0 on success */
int set_led_state(HIDInterface *pac, PacData *data) {
   int result = -1;

   memset(data, 0, PAC_DATA_HEADER_LEN);
#ifdef DEBUG
   int idx;
   for(idx=0; idx<4; idx++) {
       printf("set: byte %d: 0x%x\n", idx, ((char *)data)[idx] & 0xff);
   }
#endif
   result = hid_set_output_report(pac, PAC_PATH_IN, PAC_PATHLEN, 
                                  (char *)data, sizeof(PacData));
   if (result == HID_RET_SUCCESS) {
       result = 0;
   } else {
       printl(0, "Error result of sending output report: %d\n", result);
   }
   return result; 
}

/* Get current LED states from Pac-Drive */
/* Returns 0 on success */
int get_led_state(HIDInterface *pac, PacData *data) {
   int result = -1;

   memset(data, 0, PAC_DATA_HEADER_LEN);
   result = hid_get_input_report(pac, PAC_PATH_OUT, PAC_PATHLEN, 
                                 (char *)data, sizeof(PacData));
   if (result == HID_RET_SUCCESS) {
       result = 0;
   } else {
       printl(0, "get_led_state: FAILED\n");
   }
#ifdef DEBUG
   int idx;
   for(idx=0; idx < sizeof(PacData); idx++) {
       printf("Get_led_state: byte[%d] = 0x%x\n", idx, ((char *)data)[idx]);
   }
#endif

   return result; 
}

/* Returns NULL on error, else pac (HIDInterface) */
HIDInterface *init_pac(int board_num) {

  hid_return ret;
  HIDInterface *pac;
  HIDInterfaceMatcher matcher = { VENDOR_ID_ULTIMARC,
                                  PRODUCT_ID_PACDRIVE,
                                  (void *)match_fn_find_selected,
                                  &temp_pac_count,
                                  board_num,
                                };


  // This does a malloc under the covers.  
  // Make sure to hid_delete_HIDInterface()...  that's close_pac for us
  pac = hid_new_HIDInterface();
  if (pac == NULL) {
      printl(0, "init_pac(): failed to get a new HID interface... d'oh\n");
      return 0;
  }


  // Try up to 3 times
  // This has a matcher function that will try to only match the board number specified
  ret = hid_force_open(pac, 0, &matcher, 3);
  //printf("After open, count: %d\n", temp_pac_count);

  if (ret != HID_RET_SUCCESS) {
    printl(0, "init_pac(): hid_force_open failed with return code %d\n", ret);
    return 0;
  }

  return pac;
}

int close_pac(HIDInterface *pac) {

   usb_release_interface(pac->dev_handle, 0);
   hid_close(pac);
   // Don't think this is necessary. TODO: check libhid sources again
   hid_delete_HIDInterface(&pac);  

   return 1;
}

void cleanup_pac() {
    hid_cleanup();
}

/* Returns the number of Pac-Drives found */
/* -1 on error */
int pac_count(int interface) {
  int res;

  res = find_pacs(MAX_PACS);

  return res;
}
